#include <Rcpp.h>
using namespace Rcpp;
#include <math.h>
#include<algorithm>
#include<cmath>

// [[Rcpp::export]]
double Two_hat_I_2_cpp(NumericVector e_star, NumericMatrix X, double b1,double b2){
  int n = e_star.size();
  int p = X.ncol();
  NumericMatrix sim_X(n,n);
  NumericMatrix sim_Y(n,n);
  double diff2=0;
  NumericVector one_out(n),sumXi(n);
  double out=0;
  
  
  int i,j,k;
  for(i=0;i<n;i++){
    one_out(i)=0;
    sumXi(i)=0;
    for(j=0;j<n;j++){
      diff2=0;
      for(k=0;k<p;k++){
        diff2+=(X(i,k)-X(j,k))*(X(i,k)-X(j,k));
      }
      sim_X(i,j)=(1.0/(pow(2.0*M_PI,0.5)*b2))*exp(-diff2/(2.0*b2*b2));
      sim_Y(i,j)=(1.0/(pow(2.0*M_PI,0.5)*b1))*exp(-pow((e_star(i)-e_star(j)),2)/(2.0*b1*b1));
      one_out(i)+=(sim_X(i,j)*sim_Y(i,j));
    }
    for(k=0;k<n;k++){
      sumXi(i)+=sim_X(i,k);
    }
    sumXi(i)=sumXi(i)-sim_X(i,i);
    one_out(i)=one_out(i)-(sim_X(i,i)*sim_Y(i,i));
    out+=one_out(i)/sumXi(i);
  }
  return (1/double (n))*out; 
}




// [[Rcpp::export]]
double Two_hat_I_1_cpp(NumericVector e_star, NumericMatrix X, double b1,double b2){
  int n = e_star.size();
  int p = X.ncol();
  NumericMatrix sim_X(n,n);
  NumericMatrix sim_Y(n,n);
  double diff2=0;
  NumericVector cross_one_out(n),sumXi(n);
  double out=0;
  
  
  int i,j,k;
  for(i=0;i<n;i++){
    cross_one_out(i)=0;
    sumXi(i)=0;
    for(j=0;j<n;j++){
      diff2=0;
      for(k=0;k<p;k++){
        diff2+=pow((X(i,k)-X(j,k)),2);
      }
      sim_X(i,j)=(1.0/(pow(2.0*M_PI,0.5)*b2))*exp(-diff2/(2.0*b2*b2));
      sim_Y(i,j)=(1.0/(pow(2.0*M_PI,0.5)*pow(2,0.5)*b1))*exp(-pow((e_star(i)-e_star(j)),2)/(2.0*b1*b1*pow(2,0.5)));
    }
  }
  for(i=0;i<n;i++){
    for(k=0;k<n;k++){
      sumXi(i)+=sim_X(i,k);
      for(j=0;j<n;j++){
        cross_one_out(i)+=(sim_X(i,j)*sim_X(i,k)*sim_Y(k,j));
      }
      cross_one_out(i)=cross_one_out(i)-(sim_X(i,k)*sim_X(i,i)*sim_Y(i,k))-(sim_X(i,i)*sim_X(i,k)*sim_Y(k,i));
    }
    sumXi(i)=sumXi(i)-sim_X(i,i);
    cross_one_out(i)=cross_one_out(i)+(sim_X(i,i)*sim_X(i,i)*sim_Y(i,i));
    out+=cross_one_out(i)/pow(sumXi(i),2);
  }
  return (1/double (n))*out; 
}

// [[Rcpp::export]]
double hat_m_cpp(NumericVector x, NumericVector Y, NumericMatrix X, double b0){
  int n = Y.size();
  int p = X.ncol();
  double out1=0,out2=0,diff2=0;
  int i,k;
  for(i=0;i<n;i++){
    diff2=0;
    for(k=0;k<p;k++){
      diff2+=(x(k)-X(i,k))*(x(k)-X(i,k));
    }
    out1 += (1.0/(pow(2.0*M_PI,0.5)*b0))*exp(-diff2/(2.0*b0*b0));
    out2 += (1.0/(pow(2.0*M_PI,0.5)*b0))*exp(-diff2/(2.0*b0*b0))*Y(i);
  }
  return out2/out1;
}



// [[Rcpp::export]]
double LS_cpp(NumericVector Y, NumericMatrix X, double b0){
  int n = Y.size();
  int p = X.ncol();
  double out1=0;
  NumericVector Y_mis(n-1);
  NumericMatrix X_mis(n-1,p);
  NumericVector X_temp(n);
  int i,j,k;
  for(i=0;i<n;i++){
    for(j=0;j<n;j++){
      if(j<i){
        Y_mis(j)=Y(j);
        for(k=0;k<p;k++){
          X_mis(j,k)=X(j,k);
        }
      }
      if(j>i){
        Y_mis(j-1)=Y(j);
        for(k=0;k<p;k++){
          X_mis(j-1,k)=X(j,k);
        }
      }
    }
    X_temp = X(i,_);
    out1 += (Y(i)-hat_m_cpp(X_temp,Y_mis,X_mis,b0))*(Y(i)-hat_m_cpp(X_temp,Y_mis,X_mis,b0));
  }
  return out1;
}

// [[Rcpp::export]]
NumericVector hat_e_cpp(NumericMatrix x, NumericVector Y, NumericMatrix X, double b0){
  int n = Y.size();
  NumericVector out1(n);
  int i;
  for(i=0;i<n;i++){
    out1(i)=hat_m_cpp(x(i,_),Y,X,b0);
  }
  return out1;
}



// [[Rcpp::export]]
NumericVector Two_condKDE_cpp(NumericVector Y, NumericMatrix X, double b0, double b1,double b2){
  int n = Y.size();
  int p = X.ncol();
  NumericVector m(n),e(n);
  m = hat_e_cpp(X,Y,X,b0);
  int i,j,k;
  for(i=0;i<n;i++){
    e(i)= Y(i)-m(i);
  }
  NumericMatrix sim_X(n,n);
  NumericMatrix sim_Y(n,n);
  NumericVector emp_den(n);
  double diff2=0,sumXY,sumX;
  
  for(i=0;i<n;i++){
    for(j=0;j<n;j++){
      diff2=0;
      for(k=0;k<p;k++){
        diff2+=(X(i,k)-X(j,k))*(X(i,k)-X(j,k));
      }
      sim_X(i,j)=(1.0/(pow(2.0*M_PI,0.5)*b2))*exp(-diff2/(2.0*b2*b2));
      sim_Y(i,j)=(1.0/(pow(2.0*M_PI,0.5)*pow(2,0.5)*b1))*exp(-pow(((Y(i)-m(i))-e[j]),2)/(2.0*b1*b1*pow(2,0.5)));
    }
    sumXY=0;
    sumX=0;
    for(k=0;k<n;k++){
      sumXY += sim_X(i,k)*sim_Y(i,k);
      sumX += sim_X(i,k);
    }
    emp_den(i)=sumXY/sumX;
  }
  return emp_den;
}






